﻿using UnityEngine;
using UnityEngine.UI;
using Photon.Pun;
using Photon.Realtime;
using System;
using System.Collections.Generic;
using System.Linq;
using ExitGames.Client.Photon;
using UnityEngine.SceneManagement;

// MonoBehaviourではなくMonoBehaviourPunCallbacksを継承して、Photonのコールバックを受け取れるようにする
public class BaseSC : MonoBehaviourPunCallbacks
{
    public GameObject panelOrigin;
    public GameObject canvas;
    public GameObject atkMsgOrigin;
    public GameObject damageMsgOrigin;
    private GameObject[] panels;
    public GameObject selectedPanelOrigin;
    public GameObject[] selectedPanels;
    public GameObject rankingBoardOrigin;
    public GameObject ExitBtnText;
    public GameObject ExitBtn;

    private GameObject enemysArea;
    private bool needInit;
    private bool endDataSynced;

    private bool disconnecting;
    public GameObject playAreaOrigin;

    // Start is called before the first frame update
    void Start()
    {
        Prm.soloMode = false;
        this.gameObject.AddComponent<CommonSC>();
        // Scene読込時は必ずセーブを読み込むようにしておく
        Prm.savedataPatternType = PlayerPrefs.GetInt("panelType", 0);
        Prm.savedataSoundType = PlayerPrefs.GetInt("soundType", 0);

        needInit = true;
        Prm.mode = Mode.Start;
    }
    public void Exit()
    {
        Debug.Log("終了");
        if (PhotonNetwork.IsConnected) PhotonNetwork.Disconnect();
        SceneManager.LoadScene("Menu");
    }

    // マスターサーバーへの接続が成功した時に呼ばれるコールバック 
    public override void OnConnectedToMaster()
    {
        RoomOptions roomOptions = new RoomOptions
        {
            IsVisible = true,
            MaxPlayers = 20,
            IsOpen = true
        };
        // "room"という名前のルームに参加する（ルームが無ければ作成してから参加する）
        PhotonNetwork.JoinOrCreateRoom("room", roomOptions, TypedLobby.Default);
    }
    // マッチングが成功した時に呼ばれるコールバック
    public override void OnJoinedRoom()
    {
        Debug.Log("部屋に参加（ローカルモード："+Prm.mode);
        // 部屋のモードを取得
        if (PhotonNetwork.CurrentRoom.CustomProperties["mode"] is int mode)
        {
            if (mode <= Prm.mode) return;
            Prm.mode = mode;
            needInit = true;
        }
        ExitBtnText.SetActive(true);
        ExitBtn.SetActive(true);
    }
    public override void OnDisconnected(DisconnectCause cause)
    {
        Debug.Log("切断されました　："+cause.ToString());
        if (cause.ToString().Equals("MaxCcuReached"))
        {
            GameObject.Find("CenterText").GetComponent<Text>().text = "ERROR\r\n満員です\r\nserver join failed";
            ExitBtnText.SetActive(true);
            ExitBtn.SetActive(true);
        }
    }
    // 部屋に参加できなかった場合
    public override void OnJoinRoomFailed(short returnCode, string message)
    {
        Debug.Log("部屋が満室です");
        GameObject.Find("CenterText").GetComponent<Text>().text = "FULL\r\n満員です\r\nroom join failed";
        // exitを可能にする
        ExitBtnText.SetActive(true);
        ExitBtn.SetActive(true);
    }

    public override void OnPlayerEnteredRoom(Player newPlayer)
    {
        if (PhotonNetwork.CurrentRoom.PlayerCount >= 2 && Prm.mode == Mode.Matching)
        {
            GameObject.Find("CenterText").GetComponent<Text>().text = "[start]";
            GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = true;
        }
    }    
    public override void OnPlayerLeftRoom(Player otherPlayer)
    {
        if (PhotonNetwork.CurrentRoom.PlayerCount == 1 && Prm.mode == Mode.Matching)
        {
            GameObject.Find("CenterText").GetComponent<Text>().text = "waiting\r\nentry...";
            GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = false;
            Prm.ready = false;
            // オンラインのplayerPrmを更新
            SetPlayerOnlinePrm();
        }
    }
    public override void OnPlayerPropertiesUpdate(Player targetPlayer, Hashtable changedProps)
    {
        endDataSynced = true;
    }
    public override void OnCreatedRoom()
    {
        Debug.Log("新規に部屋を作成する");

        // ステータス更新
        SetPlayerOnlinePrm();

        // 部屋の引数を設定
        UpdateToNextMode();

        if (PhotonNetwork.CurrentRoom.PlayerCount >= 2 && Prm.mode == Mode.Matching)
        {
            GameObject.Find("CenterText").GetComponent<Text>().text = "[start]";
            GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = true;
        }
        if (PhotonNetwork.CurrentRoom.PlayerCount == 1 && Prm.mode == Mode.Matching)
        {
            GameObject.Find("CenterText").GetComponent<Text>().text = "waiting\r\nentry...";
            GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = false;
            Prm.ready = false;
            // オンラインのplayerPrmを更新
            SetPlayerOnlinePrm();
        }
    }
    public void Rematch()
    {
        Scene loadScene = SceneManager.GetActiveScene();
        if (loadScene.name == "Main")
        {
            SceneManager.LoadScene("Sub");
        }
        else
        {
            SceneManager.LoadScene("Main");
        }
    }

    // Update is called once per frame
    void Update()
    {
        if (needInit)
        {
            InitMode();
            needInit = false;
        }
        switch (Prm.mode)
        {
        case Mode.Start:
            // 接続が完了するまでは何もしない
            break;
        case Mode.Matching://待機画面
            {
                if (CheckAndDisplayOnlineStatus()) UpdateToNextMode();
                break;
            }
        case Mode.CountDown://STARTカウントダウン
            {
                Prm.timer += Time.deltaTime;
                GameObject.Find("startCountDown").GetComponent<Text>().text = ((int)((float)Prm.TIME_START_COUNT + 1 - Prm.timer)).ToString();
                if (Prm.timer >= Prm.TIME_START_COUNT)
                {
                    UpdateToNextMode();
                    // 現在のサーバー時刻を、ゲームの開始時刻に設定する
                    SetRoomStartTime();
                    return;
                }
                break;
            }
        case Mode.Play://play画面
            {
                // enemy情報を更新
                UpdateEnemys();

                // もし他が全員死んでいるなら次のシーンへ遷移
                if (AllEnemysDead())
                {
                    UpdateToNextMode();
                    return;
                }
                // 死んでる場合はrank画面更新のみ。後続処理は行わない
                if (Prm.dead)
                {
                    DispRanking();
                    return;
                }
                // 敵からの攻撃で未対応のダメージを反映する
                AddAttackToMe();

                // stock追加までの時間表示
                Prm.timer += Time.deltaTime;
                GameObject.Find("text").GetComponent<Text>().text
                    = ((int)((float)Prm.TIME_GAME_PLAY - Prm.timer)).ToString();
                // 制限時間が来たらstockを1追加
                if (Prm.timer >= Prm.TIME_GAME_PLAY)
                {
                    Prm.timer = 0;
                    Prm.stock++;
                    // オンラインのplayerPrmを更新
                    SetPlayerOnlinePrm();
                }
                break;
            }
        case Mode.End:
            {
                // 切断済みの場合は後続処理は行わない
                if (!PhotonNetwork.IsConnected) return;
                UpdateEnemys();
                // 1位が確定するまで後続の処理は行わない
                if (!DispRanking()) break;
                // 自分が勝者の場合は同期が完了するまで後続処理は行わない
                if (Prm.win && !endDataSynced) break;
                // 切断する
                if (PhotonNetwork.IsConnected && !disconnecting)
                {
                    disconnecting = true;
                    PhotonNetwork.Disconnect();
                }
                break;
            }
        }
    }

    private void InitMode()
    {
        switch (Prm.mode)
        {
            case Mode.Start:
                {
                    ExitBtnText.SetActive(false);
                    ExitBtn.SetActive(false);
                    enemysArea = GameObject.Find("EnemysArea");
                    enemysArea.SetActive(false);
                    Prm.attack = 0;
                    Prm.timer = 0;
                    Prm.combo = 0;
                    Prm.startPosX = 0;
                    Prm.startPosY = 0;
                    Prm.endPosX = 0;
                    Prm.endPosY = 0;
                    Prm.liveTime = 0;
                    Prm.stock = 6;
                    Prm.selectFlg = false;
                    Prm.ready = false;
                    Prm.dead = false;
                    Prm.win = false;
                    Prm.selectedList = new Color[0];
                    Prm.panelLiveFlg = new bool[4, 3];
                    GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = false;
                    try
                    {
                        // PhotonServerSettingsに設定した内容を使ってマスターサーバーへ接続する
                        PhotonNetwork.ConnectUsingSettings();
                    }
                    catch (Exception e) {
                        Debug.Log("接続時に何らかのエラーが発生" + e.ToString());
                    }

                    break;
                }
            case Mode.Matching:
                {
                    // オンライン用のパラメータを設定
                    SetPlayerOnlinePrm();

                    if (PhotonNetwork.CurrentRoom.PlayerCount >= 2)
                    {
                        GameObject.Find("CenterText").GetComponent<Text>().text = "[start]";
                        GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = true;
                    }
                    else
                    {
                        GameObject.Find("CenterText").GetComponent<Text>().text = "waiting\r\nentry...";
                        GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = false;
                        Prm.ready = false;
                        // オンラインのplayerPrmを更新
                        SetPlayerOnlinePrm();
                    }
                    break;
                }
            case Mode.CountDown:
                {
                    ExitBtnText.SetActive(false);
                    ExitBtn.SetActive(false);

                    Prm.ready = false;
                    Prm.timer = 0;
                    GameObject.Find("startCountDown").GetComponent<Text>().enabled = true;
                    GameObject.Find("text").GetComponent<Text>().text = "";
                    GameObject.Find("CenterText").GetComponent<Text>().enabled = false;
                    // enemyAreaを活性化
                    enemysArea.SetActive(true);

                    // 以降使わないパーツを削除
                    GameObject.Find("onlineText").GetComponent<Text>().text = "";
                    GameObject.Find("StartBtn").GetComponent<BoxCollider2D>().enabled = false;
                    //  敵情報を初期化
                    InitEnemys();
                    break;
                }
            case Mode.Play:
                {
                    Debug.Log("パネルセット");
                    GetComponent<CommonSC>().SetPanel(panels, panelOrigin,playAreaOrigin);
                    GetComponent<CommonSC>().MakeSelectedList(selectedPanels, selectedPanelOrigin);
                    Prm.timer = 0;
                    GameObject.Find("text").GetComponent<Text>().fontSize = 100;
                    // スコアとコンボとstockを初期化
                    Prm.combo = 0;
                    Prm.stock = 6;
                    Prm.attack = 0f;
                    Prm.dead = false;
                    Prm.liveTime = 0;
                    // オンラインのプレイヤーパラメータを更新
                    SetPlayerOnlinePrm();
                    // 以降使わないパーツを削除
                    GameObject.Find("startCountDown").GetComponent<Text>().text = "";
                    break;
                }
            case Mode.End:
                {
                    disconnecting = false;
                    // まだ自分は死んでいないなら死んだことにする
                    if (!Prm.dead)
                    {
                        Prm.win = true;
                        Prm.dead = true;
                        Prm.liveTime = GetLiveTime();
                        SetPlayerOnlinePrm();
                        // 不要項目を削除
                        InitForEnd();
                        endDataSynced = false;
                    }
                    else
                    {
                        // 既に死んでいる場合は一位の情報を得る為に最終状態を取得
                        UpdateEnemys();
                    }
                    break;
                }
            default:
                break;
        }
    }
    private bool AllEnemysDead()
    {
        for (int i = 0; i < Prm.enemys.Length; i++) if (!Prm.enemys[i].dead) return false;
        return true;
    }
    // 終了状態にする
    public void InitForEnd()
    {

        GameObject.Find("text").GetComponent<Text>().text = "";
        // パネル類を削除
        Destroy(GameObject.Find("PlayArea"));
        Destroy(GameObject.Find("SelectedList"));
        // 正解エリアを削除
        Destroy(GameObject.Find("PlayArea(Clone)"));

        // exitを可能にする
        ExitBtnText.SetActive(true);
        ExitBtn.SetActive(true);

    }
    // 一位まで確定している場合はtrue
    public bool DispRanking()
    {
        bool rtn=false;
        Dictionary<int, int> dict = new Dictionary<int, int>();
        int cnt = Prm.enemys.Length + 1;
        foreach (var p in Prm.enemys)
        {
            if (p.dead)
            {
                dict.Add(p.id, p.liveTime);
                cnt--;
            }
        }
        if (cnt == 1) rtn = true;
        dict.Add(PhotonNetwork.LocalPlayer.ActorNumber, Prm.liveTime);

        string bfText = "- rank -\r\n";
        var sorted = dict.OrderByDescending((x) => x.Value);
        foreach (var v in sorted)
        {
            if (PhotonNetwork.LocalPlayer.ActorNumber == v.Key) bfText += "YOU";
            if (PhotonNetwork.LocalPlayer.ActorNumber != v.Key) bfText += "   ";
            bfText += new string(' ', 3 - cnt.ToString().Length);
            bfText += cnt;
            bfText += new string(' ', 12 - v.Value.ToString().Length);
            bfText += v.Value + "ms\r\n";
            cnt++;
        }
        GameObject.Find("rankingText").GetComponent<Text>().text = bfText;
        return rtn;
    }

    private void AddAttackToMe()
    {
        float bf = 0;
        for (int i=0;i<Prm.enemys.Length;i++)
        {
            if(Prm.enemys[i].attack> Prm.enemys[i].addedAttack)
            {
                bf += Prm.enemys[i].attack - Prm.enemys[i].addedAttack;
                Prm.enemys[i].addedAttack = Prm.enemys[i].attack;
            }
        }
        // damage無しの場合は終了
        if (bf == 0f) return;
        // ダメージメッセージ
        GameObject msg = Instantiate(damageMsgOrigin);
        msg.GetComponent<DamageMsgSC>().text = "-" + (int)bf;
        msg.transform.SetParent(canvas.transform, false);


        // ダメージが残り時間より小さい場合は残り時間だけ短くして終了
        if (Prm.TIME_GAME_PLAY-Prm.timer > bf)
        {
            Prm.timer += bf;
            return;
        }
        // ダメージが残り時間よりも大きい場合ループする
        bf -= Prm.timer;
        Prm.stock++;
        while (bf > Prm.TIME_GAME_PLAY)
        {
            bf -= Prm.TIME_GAME_PLAY;
            Prm.stock++;
        }
        Prm.timer = bf;
        // stockを変えているのでオンライン更新
        SetPlayerOnlinePrm();
    }


    public void MouseDown()
    {
        Prm.selectFlg = true;
    }
    public void MouseUp()
    {
        Prm.selectFlg = false;
        // playmodeの場合は正誤判定を実施
        if (Prm.mode == Mode.Play)CheckAnswer();
    }

    // 選択済み色の表示用パネルのアップデート(末尾を変更)
    private void UpdateSelectedList()
    {
        selectedPanels[Prm.selectedList.Length - 1].GetComponent<Renderer>().material.color
            = Prm.selectedList[Prm.selectedList.Length - 1];
    }

    public void CheckAnswer()
    {
        switch (GetComponent<CommonSC>().CheckAnswerMain())
        {
            case -1:// 誤り
                    // 初期化して問題再作成
                GetComponent<CommonSC>().MakeAnswerPanel(false);
                GetComponent<CommonSC>().MakeSelectListAnime(false);
                GameObject.Find("SelectedList").GetComponent<SelectedListSC>().DestroyAllChilds();
                GetComponent<CommonSC>().SetPanel(panels, panelOrigin,playAreaOrigin);
                GetComponent<CommonSC>().MakeSelectedList(selectedPanels, selectedPanelOrigin);
                // コンボリセット
                Prm.combo = 0;
                break;
            case 1:// 完成
                // stockを一つ削る
                Prm.stock--;
                // もし全消ししている場合はメッセージを表示した上で攻撃する
                if (Prm.stock <= 0)
                {
                    Prm.stock = 0;
                    GameObject msg = Instantiate(atkMsgOrigin);
                    msg.GetComponent<AtkMsgSC>().text = "Erase all!\r\n\r\n\r\n";
                    msg.transform.SetParent(canvas.transform, false);
                    // 攻撃(1サイクル分の攻撃)
                    Prm.attack += Prm.TIME_GAME_PLAY;
                }

                // オンラインのplayerPrmを更新
                SetPlayerOnlinePrm();

                // 初期化して問題再作成
                GetComponent<CommonSC>().MakeAnswerPanel(true);
                GetComponent<CommonSC>().MakeSelectListAnime(true);
                GameObject.Find("SelectedList").GetComponent<SelectedListSC>().DestroyAllChilds();
                GetComponent<CommonSC>().SetPanel(panels, panelOrigin,playAreaOrigin);
                GetComponent<CommonSC>().MakeSelectedList(selectedPanels, selectedPanelOrigin);
                // スコア＆コンボ加算
                AddCombo();
                break;
            case 0:// 未完成の場合
                   // 何もしない
                break;
        }
    }
    private void AddCombo()
    {
        Prm.combo++;
        // コンボメッセージ
        GameObject msg = Instantiate(atkMsgOrigin);
        msg.GetComponent<AtkMsgSC>().text = "combo " + Prm.combo+ "\r\n";
        msg.transform.SetParent(canvas.transform, false);

        // コンボが2以上の場合、アタックを追加
        if (Prm.combo >= 2) AddAttack();
    }

    private void AddAttack()
    {
        // コンボが1以下の場合処理しない
        if (Prm.combo <= 1) return;
        // 攻撃力はコンボ数/参加人数で決める
        float attackBase = (float)Prm.combo;
        if (attackBase > Prm.ATTACK_COMBO_MAX) attackBase = (float)Prm.ATTACK_COMBO_MAX;
        // 生きている人数を確認（0の場合は終了）
        int num = 0;
        for (int i = 0; i < Prm.enemys.Length; i++) if (!Prm.enemys[i].dead) num++;
        if (num <= 0) return;
        Prm.attack += (float)Math.Pow(Prm.ATTACK_DAMAGE_BASE, attackBase) / (float)num;
        SetPlayerOnlinePrm();
    }


    // オンライン情報を表示する 2人以上存在する状態で、全員Readyならtrueを返す
    public bool CheckAndDisplayOnlineStatus()
    {
        bool flg = true;
        // 未接続の場合はその旨だけ表示して終了
        if (!PhotonNetwork.IsConnected || !PhotonNetwork.InRoom)
        {
            GameObject.Find("onlineText").GetComponent<Text>().text = "connecting...";
            return false;
        }
        string bfText = "\r\n - Player List - \r\n\r\n";
        int i = 1;
        foreach (var p in PhotonNetwork.PlayerList)
        {
            if (p.CustomProperties["ready"] is bool ready)
            {
                bfText += "[";
                if (PhotonNetwork.LocalPlayer.ActorNumber == p.ActorNumber) bfText += "YOU ";
                bfText = bfText + "Player"+i+ "] ";
                if (ready) bfText += " - READY -\r\n";
                if (!ready) bfText += "  - WAIT -\r\n";
                if (!ready) flg = false;
            }
            else {
                bfText += "[";
                if (PhotonNetwork.LocalPlayer.ActorNumber == p.ActorNumber) bfText += "YOU ";
                bfText = bfText + "Player" + i + "] ";
                bfText = bfText + p.ActorNumber + "connect...\r\n";
                flg = false;
            }
            i++;
        }
        GameObject.Find("onlineText").GetComponent<Text>().text = bfText;
        //Debug.Log("プレイヤー数チェック：" + i);
        if (i < 2) flg = false;
        return flg;

    }


    // ルームのmodeを設定する
    public void UpdateToNextMode()
    {
        Prm.mode++;
        // カスタムプロパティの追加
        ExitGames.Client.Photon.Hashtable roomHash = new ExitGames.Client.Photon.Hashtable
        {
            { "mode", Prm.mode }
        };
        // カウントダウンに入る場合、その時点で部屋への新規参加を締め切るorこの後の参加者は観戦モードにする
        // TODO
        if(Prm.mode == Mode.CountDown)PhotonNetwork.CurrentRoom.IsOpen = false;
        // 更新
        PhotonNetwork.CurrentRoom.SetCustomProperties(roomHash);
        needInit = true;
    }


    // 部屋のステータスがアップデートされた時に実施
    public override void OnRoomPropertiesUpdate(Hashtable propertiesThatChanged)
    {
        if (!PhotonNetwork.IsConnected || !PhotonNetwork.InRoom) return;
        if (PhotonNetwork.CurrentRoom.CustomProperties["mode"] is int mode)
        {
            Debug.Log("サーバー上のモード更新：" + mode);
            if (mode <= Prm.mode) return;
            Prm.mode = mode;
            needInit = true;
        }
    }
    public void SetRoomStartTime()
    {
        // カスタムプロパティの追加
        ExitGames.Client.Photon.Hashtable roomHash = new ExitGames.Client.Photon.Hashtable
        {
            { "startTime", PhotonNetwork.ServerTimestamp }
        };
        PhotonNetwork.CurrentRoom.SetCustomProperties(roomHash);
    }
    public int GetRoomStartTime()
    {
        if (!PhotonNetwork.IsConnected || !PhotonNetwork.InRoom) return 0;
        if (PhotonNetwork.CurrentRoom.CustomProperties["startTime"] is int time) return time;
        return 0;
    }
    public void UpdateEnemys()
    {
        // 先に全て死亡設定にしておく
        for (int i = 0; i < Prm.enemys.Length; i++)Prm.enemys[i].dead = true;
        // IDが一致するplayer情報を反映
        foreach (var p in PhotonNetwork.PlayerListOthers)
        {
            for(int i = 0; i < Prm.enemys.Length; i++)
            {
                if (Prm.enemys[i].id == p.ActorNumber)
                {
                    if (
                        p.CustomProperties["dead"] is bool dead &&
                        p.CustomProperties["liveTime"] is int liveTime &&
                        p.CustomProperties["stock"] is int stock &&
                        p.CustomProperties["attack"] is float attack)
                    {
                        Prm.enemys[i].dead = dead;
                        Prm.enemys[i].liveTime = liveTime;
                        Prm.enemys[i].stock = stock;
                        Prm.enemys[i].attack = attack;
                    }
                    break;
                }
            }
        }
    }



    public void InitEnemys()
    {
        int length = PhotonNetwork.PlayerListOthers.Length;
        Prm.enemys = new Enemys[length];
        int i = 0;
        foreach (var p in PhotonNetwork.PlayerListOthers)
        {
            if (
                p.CustomProperties["dead"] is bool dead &&
                p.CustomProperties["liveTime"] is int liveTime &&
                p.CustomProperties["stock"] is int stock &&
                p.CustomProperties["attack"] is float attack)
            {
                Prm.enemys[i] = new Enemys
                {
                    dead = dead,
                    liveTime = liveTime,
                    stock = stock,
                    attack = attack,
                    addedAttack = 0,
                    id = p.ActorNumber
                };
            }
            else
            {
                Debug.Log(p.ActorNumber + "パラメータが不正");
            }
            i++;
        }
        for (int n=i;n<length;n++) {
            Prm.enemys[n] = new Enemys
            {
                dead = true,
                liveTime = 0,
                stock = 12,
                attack = 0,
                addedAttack = 0,
                id = -1
            };
        }
    }

    // オンライン情報を更新する
    public void SetPlayerOnlinePrm()
    {
        var hashtable = new ExitGames.Client.Photon.Hashtable
        {
            ["ready"] = Prm.ready,
            ["dead"] = Prm.dead,
            ["liveTime"] = Prm.liveTime,
            ["stock"] = Prm.stock,
            ["attack"] = Prm.attack
        };
        PhotonNetwork.LocalPlayer.SetCustomProperties(hashtable);
    }

    // 開始時間を元に生存時間を取得する
    public int GetLiveTime()
    {
        int start = GetRoomStartTime();
        int now = PhotonNetwork.ServerTimestamp;
        int diff= 0;
        if (start > now)
        {
            diff = now - start;
        }
        else
        {
            // オーバーフローしている場合は上下分けて取得してから合算
            diff += now - int.MinValue;
            diff += int.MaxValue - start;
        }
        return diff;
    }
}
