﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayAreaHexSC : MonoBehaviour
{
    // mode = 0 ゲームエリアとして使用
    // mode = 1 解答エリアとして使用
    public int mode;

    public GameObject[] panels;
    public bool[][] panelLiveHexFlg;

    // 最初に選択したパネルに準じてガギ音を切り替える為に選択済み配列を記憶する


    // Start is called before the first frame update
    void Start()
    {
        // 子のパネルを全てリストに紐づける
        panels = new GameObject[12];
        foreach (Transform n in this.transform.Find("panels").transform)
        {
            panels[n.gameObject.transform.GetComponent<PanelScriptHex>().no] =n.gameObject;
        }
        // GINGAMANパターンをランダムに取得してNoに反映する。
        int[] pattern = GingaPatternHex.random();
        for (int i = 0; i < panels.Length; i++)panels[i].transform.GetComponent<PanelScriptHex>().no = pattern[i];
        // ガンギマ色を設定
        Prm.colorABCD = new Color[4];
        float rnd = (float)(UnityEngine.Random.Range(0, 1000f)) / 1000f;
        for (int i = 0; i < Prm.colorABCD.Length; i++)
        {
            Prm.colorABCD[i] = UnityEngine.Color.HSVToRGB(rnd, 0.70f, 1);
            rnd += 1f / (Prm.colorABCD.Length);
            if (rnd > 1f) rnd -= 1f;
        }
        // パネルに模様を付ける場合のパターン用乱数を取得
        int patternRnd = (int)UnityEngine.Random.Range(0, 4);
        // 各パネルに色とパターンを反映
        for (int i = 0; i < panels.Length; i++)
        {
            panels[i].transform.GetComponent<PanelScriptHex>().patternRnd = patternRnd;
            panels[i].transform.GetComponent<PanelScriptHex>().InitPanelColor();
            panels[i].transform.GetComponent<PanelScriptHex>().InitPanelPattern();
        }
        panelLiveHexFlg = new bool[][] {
            new bool[3]{true,true,true},
            new bool[4]{true, true, true, true },
            new bool[3]{true,true,true},
            new bool[2]{true,true},
        };
    }

    // Update is called once per frame
    void Update()
    {
        if (mode == 1)
        {
            if (transform.position.y > -3.3f)
            {
                transform.Translate(Time.deltaTime * 7.5f, -Time.deltaTime * 18, 0);
                transform.localScale = new Vector3(
                    transform.localScale.x - Time.deltaTime * 3f
                    , transform.localScale.y - Time.deltaTime * 3f
                    , 0);
                transform.GetComponent<LineRenderer>().startWidth -= Time.deltaTime / 2f;
                transform.GetComponent<LineRenderer>().endWidth -= Time.deltaTime / 2f;
            }
            else
            {
                transform.position = new Vector3(1.5f, -3.3f, 0);
            }
        }
    }
    public void DestroyAllPanels()
    {
        foreach (Transform n in this.transform.Find("panels").transform)
        {
            GameObject.Destroy(n.gameObject);
        }
    }

    // アンサーモードへの切り替え
    public void ChangeToAnswerMode(bool answered)
    {
        // PlayArea(Clone)が既に存在する場合は削除
        Destroy(GameObject.Find("PlayAreaHex(Clone)"));
        // 自身の名前を変更
        this.gameObject.name = "PlayAreaHex(Clone)";

        // 全てのパネルの色を暗くする
        ChangeDarkPanels();
        // lineを使用していない場合（活性化していない場合）が有り得るので有効化しておく
        this.transform.GetComponent<LineRenderer>().enabled = true;
        // 正解、不正解でlineとStartPointの色を変更する
        Color bf = Color.green;
        if (!answered) bf = Color.red;
        this.transform.Find("StartPoint").gameObject.GetComponent<Renderer>().material.color = bf;
        this.transform.GetComponent<LineRenderer>().startColor = bf;
        this.transform.GetComponent<LineRenderer>().endColor = bf;

        // playAreaの解答が誤っている場合のみStartPointとLineのPosを修正する
        if (!answered)
        {
            // lineを12頂点化
            GetComponent<LineRenderer>().positionCount = 12;
            // panel情報を元に解答のラインを作成する
            var childScripts = GetComponentsInChildren<PanelScriptHex>();
            foreach (var item in childScripts)
            {
                // StartPointのPosを設定
                if (item.no == 0)
                {
                    transform.Find("StartPoint").gameObject.transform.position = new Vector3(item.transform.position.x, item.transform.position.y, 0);
                }
                // lineの各Posを設定
                GetComponent<LineRenderer>().SetPosition(item.no, new Vector3(item.transform.position.x, item.transform.position.y, 0));
            }
        }
        // ソート順序を整理
        var childSprits = GetComponentsInChildren<SpriteRenderer>();
        foreach (var item in childSprits)
        {
            item.sortingOrder -= 15;
        }
        transform.Find("StartPoint").gameObject.GetComponent<SpriteRenderer>().sortingOrder = -9;
        GetComponent<LineRenderer>().sortingOrder = -9;
        // 当たり判定を無効化
        var childColliders = transform.Find("panels").transform.GetComponentsInChildren<PolygonCollider2D>();
        foreach (var item in childColliders) item.enabled = false;

        // Answerにモード変更
        mode = 1;
    }

    // 全てのパネルを暗くする(正しく動作しない)
    public void ChangeDarkPanels()
    {
        foreach (Transform n in this.transform.Find("panels").transform)
        {
            Color.RGBToHSV(n.gameObject.transform.GetComponent<Renderer>().material.color, out float H, out float _, out float _);
            n.gameObject.transform.GetComponent<Renderer>().material.color = Color.HSVToRGB(H, 0.70f, 0.5f);
        }
    }


}
