﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayAreaSC : MonoBehaviour
{
    // mode = 0 ゲームエリアとして使用
    // mode = 1 解答エリアとして使用
    public int mode;


    // Start is called before the first frame update
    void Start()
    {
    }

    // Update is called once per frame
    void Update()
    {
        if (mode == 1)
        {
            if(transform.position.y > -2.5f)
            {
                transform.Translate(0, -Time.deltaTime * 18, 0);
                transform.localScale = new Vector3(
                    transform.localScale.x - Time.deltaTime * 2
                    , transform.localScale.y - Time.deltaTime * 2
                    , 0);
                transform.GetComponent<LineRenderer>().startWidth -= Time.deltaTime / 3;
                transform.GetComponent<LineRenderer>().endWidth -= Time.deltaTime / 3;
            }
            else
            {
                transform.position = new Vector3(1.5f,-2.5f, 0);
            }
        }
    }
    public void DestroyAllPanels()
    {
        foreach (Transform n in this.transform.Find("panels").transform)
        {
            GameObject.Destroy(n.gameObject);
        }
    }

    // アンサーモードへの切り替え
    public void ChangeToAnswerMode(bool answered)
    {
        // PlayArea(Clone)が既に存在する場合は削除
        Destroy(GameObject.Find("PlayArea(Clone)"));
        // 自身の名前を変更
        this.gameObject.name = "PlayArea(Clone)";

        // 全てのパネルの色を暗くする
        ChangeDarkPanels();
        // lineを使用していない場合（活性化していない場合）が有り得るので有効化しておく
        this.transform.GetComponent<LineRenderer>().enabled = true;
        // 正解、不正解でlineとStartPointの色を変更する
        Color bf = Color.green;
        if (!answered) bf = Color.red;
        this.transform.Find("StartPoint").gameObject.GetComponent<Renderer>().material.color = bf;
        this.transform.GetComponent<LineRenderer>().startColor = bf;
        this.transform.GetComponent<LineRenderer>().endColor = bf;

        // playAreaの解答が誤っている場合のみStartPointとLineのPosを修正する
        if (!answered)
        {
            // lineを12頂点化
            GetComponent<LineRenderer>().positionCount = 12;
            // panel情報を元に解答のラインを作成する
            var childScripts = GetComponentsInChildren<PanelScript>();
            foreach (var item in childScripts)
            {
                // StartPointのPosを設定
                if (item.no == 0)
                {
                    transform.Find("StartPoint").gameObject.transform.position = new Vector3(item.x - 1.5f, item.y - 1, 0);
                }
                // lineの各Posを設定
                GetComponent<LineRenderer>().SetPosition(item.no, new Vector3(item.x - 3, item.y - 2, 0));
            }
        }
        // ソート順序を整理
        var childSprits = GetComponentsInChildren<SpriteRenderer>();
        foreach (var item in childSprits)
        {
            item.sortingOrder -= 15;
        }
        transform.Find("StartPoint").gameObject.GetComponent<SpriteRenderer>().sortingOrder = -9;
        GetComponent<LineRenderer>().sortingOrder = -9;
        // 当たり判定を無効化
        var childColliders = GetComponentsInChildren<BoxCollider2D>();
        foreach (var item in childColliders) item.enabled = false;

        // Answerにモード変更
        mode = 1;
    }

    // 全てのパネルを暗くする(正しく動作しない)
    public void ChangeDarkPanels()
    {
        foreach (Transform n in this.transform.Find("panels").transform)
        {
            Color.RGBToHSV(n.gameObject.transform.GetComponent<Renderer>().material.color, out float H, out float _, out float _);
            n.gameObject.transform.GetComponent<Renderer>().material.color = Color.HSVToRGB(H, 0.70f, 0.5f);
        }
    }


}
