﻿using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class StockAreaSC : MonoBehaviour
{
    // stockの上限
    public const int deathLimit = 12;
    // stockがフルになってから死ぬまでのラグタイム
    public const float deathLagTime = 1f;
    private float time;
    // default状態とのサイズ比率、座標比率を保存
    private float diffPosX;
    private float diffPosY;
    private float diffSizeX;
    private float diffSizeY;

    // 自分自身の場合は-1を設定
    public int id;
    private int inputStock;
    private bool dead;
    public GameObject stockPanelOrigin;
    private Color lineColorBK;
    private List<GameObject> stockPanels;

    // Start is called before the first frame update
    void Start()
    {
        stockPanels = new List<GameObject>();
        lineColorBK = GameObject.Find("deathLine").transform.GetComponent<SpriteRenderer>().material.color;
        diffPosX = transform.position.x;
        diffPosY = transform.position.y + transform.localScale.y / 2.2f;
        diffSizeX = transform.localScale.x;
        diffSizeY = transform.localScale.y/8.8888f;
    }

    // Update is called once per frame
    void Update()
    {
        // ゲームモード以降でないと処理は行わない
        if (Prm.mode < Mode.Play) return;
        // 勝利確定している場合、自分のStockに対しては何もしない
        if (Prm.win && id ==-1) return;
        // 設定したIDからデータ取得
        if (id == -1)
        {
            inputStock = Prm.stock;
            dead = Prm.dead;
        }
        else
        {
            // 対象が存在しない場合はデストロイする
            if (Prm.enemys.Length < id + 1)
            {
                Destroy(this.gameObject);
            }
            else
            {
                inputStock = Prm.enemys[id].stock;
                dead = Prm.enemys[id].dead;
            }
        }
        // 死んでいる場合、画面下に零れ落ちたobjを消処理だけして終了
        if (dead)
        {
            // ゲージをdead状態にする
            transform.GetComponent<SpriteRenderer>().material.color = Color.black;
            // 全てのStockの重力をONにする
            for (int i = 0; i < stockPanels.Count; i++) stockPanels[i].transform.GetComponent<Rigidbody2D>().simulated = true;
            // 画面外下部の物を消す
            DestroyUnderGroundStocks();
            return;
        }
        // 落下処理
        for (int i = 0; i < stockPanels.Count; i++)
        {
            if(stockPanels[i].transform.position.y < -4f + i * 0.74f)
            {
                stockPanels[i].transform.GetComponent<Rigidbody2D>().simulated = false;
                stockPanels[i].transform.position = new Vector3(
                        stockPanels[i].transform.position.x,
                        -4f + i * 0.74f,
                        stockPanels[i].transform.position.z);
            }
        }

        while (stockPanels.Count != inputStock)
        {
            if (stockPanels.Count < inputStock) AddStock();
            if (stockPanels.Count > inputStock) DestroyStock();
        }
        // Limitを超えた場合、死までのカウントダウンを始める
        DeathCountDown();
    }
    private void DestroyUnderGroundStocks()
    {
        for (int i = 0; i < stockPanels.Count; i++)
        {
            if (- 5f > stockPanels[i].transform.position.y)
            {
                Destroy(stockPanels[i].transform.gameObject);
                stockPanels.RemoveAt(i);
            }
        }
    }


    private void DeathCountDown()
    {
        // この処理は自プレイヤーに対してのみ行う
        if (id != -1) return;

        var childRenderers = this.gameObject.transform.GetComponentsInChildren<SpriteRenderer>();
        // Limitに到達していない場合は処理中断
        if (deathLimit > stockPanels.Count)
        {
            // タイマーが進んでいる場合はパネル色とタイマーをリセット
            if (time != 0f)
            {
                time = 0f;
                foreach (var item in childRenderers)
                {
                    item.material.color = Color.white;
                }
                this.gameObject.transform.Find("deathLine").transform.GetComponent<SpriteRenderer>().material.color = lineColorBK;
            }
            return;
        }
        time += Time.deltaTime;
        // 色を少しずつ黒くしていく
        foreach (var item in childRenderers)
        {
            Color.RGBToHSV(item.material.color, out float H, out float S, out float V);
            V -= Time.deltaTime * 0.3f;
            if (S < 0f) S = 0f;
            item.material.color = UnityEngine.Color.HSVToRGB(H, S, V);
        }

        // 死後のラグタイムを超えた場合
        if (time >= deathLagTime)
        {
            dead = true;
            Prm.dead = true;

            // オンラインのplayerPrmを更新後、画面を終了状態に遷移する
            Prm.liveTime = GameObject.Find("BaseSC").GetComponent<BaseSC>().GetLiveTime();
            GameObject.Find("BaseSC").GetComponent<BaseSC>().SetPlayerOnlinePrm();
            GameObject.Find("BaseSC").GetComponent<BaseSC>().InitForEnd();
        }
    }
    // stockを追加する
    private void AddStock()
    {
        float y = diffPosY;
        if (stockPanels.Count != 0)y = stockPanels[stockPanels.Count - 1].transform.position.y + 1f;
        if (y < diffPosY) y = diffPosY;
        GameObject obj = Instantiate(stockPanelOrigin);
        // 画面外に配置。一つ手前のパネルよりは上になるように配置
        obj.transform.position = new Vector3(diffPosX, y, 0);
        obj.transform.localScale = new Vector3(diffSizeX*0.95f,diffSizeY*0.7f,0);
        // PlayArea-panelsの子供にしておく
        obj.transform.parent = this.gameObject.transform;
        // リストに追加
        stockPanels.Add(obj);
    }
    // 最下部のストックを削除する
    private void DestroyStock()
    {
        Destroy(stockPanels[0]);
        stockPanels.RemoveAt(0);
        foreach(GameObject stock in stockPanels) stock.transform.GetComponent<Rigidbody2D>().simulated = true;
    }

}
